@@ -185,8 +185,7 @@ class HalDeviceProbe(DeviceProbe):
         if devobject.QueryCapability("video4linux"):
             location = devobject.GetProperty("video4linux.device")
             info = devobject.GetProperty("info.product")
-            srcdev = V4LSourceDeviceFactory(device=location,
-                                            displayname=info)
+            srcdev = V4LSourceDeviceFactory(name=info, device=location)
             self.debug("Valid source %r", dev)
             self._sources['video'][device_udi] = srcdev
             self.emit("device-added", srcdev)
@@ -198,15 +197,14 @@ class HalDeviceProbe(DeviceProbe):
                 info = devobject.GetProperty("alsa.card_id")
                 if alsatype == "capture":
                     self.debug("Valid source %r", dev)
-                    self._sources['audio'][device_udi] = AlsaSourceDeviceFactory(card=card,
-                                                                  device=device,
-                                                                  displayname=info)
+                    self._sources['audio'][device_udi] = \
+                            AlsaSourceDeviceFactory(name=info, card=card,
+                                    device=device)
                     self.emit("device-added", self._sources['audio'][device_udi])
                 elif alsatype == "playback":
                     self.debug("Valid sink %r", dev)
-                    self._sinks[device_udi] = AlsaSinkDeviceFactory(card=card,
-                                                              device=device,
-                                                              displayname=info)
+                    self._sinks[device_udi] = AlsaSinkDeviceFactory(name=info,
+                            card=card, device=device)
                     self.emit("device-added", self._sinks[device_udi])
 
     def _deviceAddedCb(self, device_udi, *unused_args):
@@ -243,7 +241,8 @@ class AlsaSourceDeviceFactory(SourceDeviceFactory):
     """ObjectFactory for Alsa source devices"""
 
     def __init__(self, card, device, *args, **kwargs):
-        SourceDeviceFactory.__init__(self, *args, **kwargs)
+        # FIXME: create an URI handler for this
+        SourceDeviceFactory.__init__(self, "alsasrc://", *args, **kwargs)
         self.is_audio = True
         self._card = card
         self._device = device
@@ -253,9 +252,8 @@ class AlsaSourceDeviceFactory(SourceDeviceFactory):
         alsa.set_property("device", "hw:%d,%d" % (self._card, self._device))
         return alsa
 
-    def __repr__(self):
-        return "<%s: %s [hw:%s,%s]>" % (self.__class__.__name__,
-                                        self.displayname or self.name,
+    def __str__(self):
+        return "<%s: %s [hw:%s,%s]>" % (self.__class__.__name__, self.name,
                                         self._card, self._device)
 
 class AlsaSinkDeviceFactory(SinkDeviceFactory):
@@ -272,9 +270,8 @@ class AlsaSinkDeviceFactory(SinkDeviceFactory):
         alsa.set_property("device", "hw:%d,%d" % (self._card, self._device))
         return alsa
 
-    def __repr__(self):
-        return "<%s: %s [hw:%s,%s]>" % (self.__class__.__name__,
-                                        self.displayname or self.name,
+    def __str__(self):
+        return "<%s: %s [hw:%s,%s]>" % (self.__class__.__name__, self.name,
                                         self._card, self._device)
 
 
@@ -283,7 +280,8 @@ class V4LSourceDeviceFactory(SourceDeviceFactory):
     """ObjectFactory for Video4Linux source devices"""
 
     def __init__(self, device, *args, **kwargs):
-        SourceDeviceFactory.__init__(self, *args, **kwargs)
+        # FIXME: create an URI handler for this
+        SourceDeviceFactory.__init__(self, "v4lsrc://", *args, **kwargs)
         self.is_video = True
         self._device = device
         self.__probed = False
