@@ -26,6 +26,7 @@ Main GTK+ window
 import os
 import gtk
 import gobject
+gobject.threads_init()
 import gst
 import gst.pbutils
 from urllib import unquote
@@ -41,9 +42,9 @@ from gettext import gettext as _
 
 from pitivi.log.loggable import Loggable
 
-from timeline import Timeline
-from projecttabs import ProjectTabs
-from viewer import PitiviViewer
+from pitivi.ui.timeline import Timeline
+from pitivi.ui.projecttabs import ProjectTabs
+from pitivi.ui.viewer import PitiviViewer
 from pitivi.configure import pitivi_version, APPNAME, get_pixmap_dir, \
      get_global_pixmap_dir
 from pitivi.ui import dnd
@@ -52,6 +53,11 @@ from pitivi.action import ViewAction
 from pitivi.settings import GlobalSettings
 from pitivi.receiver import receiver, handler
 import pitivi.formatters.format as formatter
+from pitivi.sourcelist import SourceListError
+from pitivi.ui.sourcelist import SourceList
+from pitivi.ui.common import beautify_factory
+from pitivi.utils import beautify_length
+from pitivi.ui.zoominterface import Zoomable
 
 if HAVE_GCONF:
     D_G_INTERFACE = "/desktop/gnome/interface"
@@ -103,6 +109,14 @@ GlobalSettings.addConfigOption('lastExportFolder',
                                 key="last-export-folder",
                                 environment="PITIVI_EXPORT_FOLDER",
                                 default=os.path.expanduser("~"))
+GlobalSettings.addConfigOption('elementSettingsDialogWidth',
+    section='export',
+    key='element-settings-dialog-width',
+    default = 620)
+GlobalSettings.addConfigOption('elementSettingsDialogHeight',
+    section='export',
+    key='element-settings-dialog-height',
+    default = 460)
 
 def supported(info):
     return formatter.can_handle_location(info[1])
@@ -110,12 +124,14 @@ def supported(info):
 def create_stock_icons():
     """ Creates the pitivi-only stock icons """
     gtk.stock_add([
-            ('pitivi-render', 'Render', 0, 0, 'pitivi'),
-            ('pitivi-split', 'Split', 0, 0, 'pitivi'),
-            ('pitivi-unlink', 'Unlink', 0, 0, 'pitivi'),
-            ('pitivi-link', 'Link', 0, 0, 'pitivi'),
-            ('pitivi-ungroup', 'Ungroup', 0, 0, 'pitivi'),
-            ('pitivi-group', 'Group', 0, 0, 'pitivi'),
+            ('pitivi-render', _('Render'), 0, 0, 'pitivi'),
+            ('pitivi-split', _('Split'), 0, 0, 'pitivi'),
+            ('pitivi-unlink', _('Unlink'), 0, 0, 'pitivi'),
+            # Translators: This is an action, the title of a button
+            ('pitivi-link', _('Link'), 0, 0, 'pitivi'),
+            ('pitivi-ungroup', _('Ungroup'), 0, 0, 'pitivi'),
+            # Translators: This is an action, the title of a button
+            ('pitivi-group', _('Group'), 0, 0, 'pitivi'),
             ])
     pixmaps = {
         "pitivi-render" : "pitivi-render-24.png",
@@ -156,7 +172,6 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.error_dialogbox = None
         self.settings = instance.settings
         self.is_fullscreen = self.settings.mainWindowFullScreen
-        self.missing_plugins = []
         self.timelinepos = 0
         self.prefsdialog = None
         create_stock_icons()
@@ -164,6 +179,28 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self._createUi(instance)
         self.app = instance
 
+        self.app.projectManager.connect("new-project-loading",
+                self._projectManagerNewProjectLoadingCb)
+        self.app.projectManager.connect("new-project-loaded",
+                self._projectManagerNewProjectLoadedCb)
+        self.app.projectManager.connect("new-project-failed",
+                self._projectManagerNewProjectFailedCb)
+        self.app.projectManager.connect("save-project-failed",
+                self._projectManagerSaveProjectFailedCb)
+        self.app.projectManager.connect("project-saved",
+                self._projectManagerProjectSavedCb)
+        self.app.projectManager.connect("closing-project",
+                self._projectManagerClosingProjectCb)
+        self.app.projectManager.connect("project-closed",
+                self._projectManagerProjectClosedCb)
+        self.app.projectManager.connect("missing-uri",
+                self._projectManagerMissingUriCb)
+
+        self.app.action_log.connect("commit", self._actionLogCommit)
+        self.app.action_log.connect("undo", self._actionLogUndo)
+        self.app.action_log.connect("redo", self._actionLogRedo)
+        self.app.action_log.connect("cleaned", self._actionLogCleaned)
+
         # if no webcams available, hide the webcam action
         self.app.deviceprobe.connect("device-added", self._deviceChangeCb)
         self.app.deviceprobe.connect("device-removed", self._deviceChangeCb)
@@ -215,6 +252,12 @@ class PitiviMainWindow(gtk.Window, Loggable):
              None, _("Edit the project settings"), self._projectSettingsCb),
             ("RenderProject", 'pitivi-render' , _("_Render project"),
              None, _("Render project"), self._recordCb),
+            ("Undo", gtk.STOCK_UNDO,
+             _("_Undo"),
+             "<Ctrl>Z", _("Undo the last operation"), self._undoCb),
+            ("Redo", gtk.STOCK_REDO,
+             _("_Redo"),
+             "<Ctrl>Y", _("Redo the last operation that was undone"), self._redoCb),
             ("PluginManager", gtk.STOCK_PREFERENCES ,
              _("_Plugins..."),
              None, _("Manage plugins"), self._pluginManagerCb),
@@ -265,6 +308,8 @@ class PitiviMainWindow(gtk.Window, Loggable):
 
         # deactivating non-functional actions
         # FIXME : reactivate them
+        save_action = self.actiongroup.get_action("SaveProject")
+        save_action.set_sensitive(False)
 
         for action in self.actiongroup.list_actions():
             action_name = action.get_name()
@@ -272,6 +317,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
                 self.render_button = action
                 # this will be set sensitive when the timeline duration changes
                 action.set_sensitive(False)
+                action.props.is_important = True
             elif action_name == "ImportfromCam":
                 self.webcam_button = action
                 action.set_sensitive(False)
@@ -282,7 +328,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
                 action.set_visible(False)
             elif action_name in [
                 "ProjectSettings", "Quit", "File", "Edit", "Help", "About",
-                "View", "FullScreen", "FullScreenAlternate", "ImportSources",
+                "View", "FullScreen", "FullScreenAlternate",
                 "ImportSourcesFolder", "PluginManager", "PlayPause",
                 "Project", "FrameForward", "FrameBackward",
                 "ShowHideMainToolbar", "ShowHideTimelineToolbar", "Library",
@@ -290,10 +336,16 @@ class PitiviMainWindow(gtk.Window, Loggable):
                 "SecondForward", "SecondBackward", "EdgeForward",
                 "EdgeBackward", "Preferences"]:
                 action.set_sensitive(True)
-            elif action_name in ["SaveProject", "SaveProjectAs",
-                    "NewProject", "OpenProject"]:
+            elif action_name in ["NewProject", "SaveProjectAs", "OpenProject"]:
                 if instance.settings.fileSupportEnabled:
                     action.set_sensitive(True)
+            elif action_name == "SaveProject":
+                if instance.settings.fileSupportEnabled:
+                    action.set_sensitive(True)
+                action.props.is_important = True
+            elif action_name == "Undo":
+                action.set_sensitive(True)
+                action.props.is_important = True
             else:
                 action.set_sensitive(False)
 
@@ -306,8 +358,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
     def _createUi(self, instance):
         """ Create the graphical interface """
         self.set_title("%s v%s" % (APPNAME, pitivi_version))
-        self.set_geometry_hints(min_width=800, min_height=480)
-        self.connect("destroy", self._destroyCb)
+        self.connect("delete-event", self._deleteCb)
         self.connect("configure-event", self._configureCb)
 
         # main menu & toolbar
@@ -324,10 +375,13 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.timeline = Timeline(instance, self.uimanager)
         self.timeline.project = self.project
 
-        vpaned.pack2(self.timeline, resize=True, shrink=False)
+        vpaned.pack2(self.timeline, resize=False, shrink=False)
         hpaned = gtk.HPaned()
-        vpaned.pack1(hpaned, resize=False, shrink=True)
-        self.projecttabs = ProjectTabs(instance, self.uimanager)
+        vpaned.pack1(hpaned, resize=True, shrink=False)
+        self.projecttabs = ProjectTabs()
+
+        self.sourcelist = SourceList(instance, self.uimanager)
+        self.projecttabs.append_page(self.sourcelist, gtk.Label(_("Clip Library")))
         self._connectToSourceList()
 
         hpaned.pack1(self.projecttabs, resize=True, shrink=False)
@@ -358,6 +412,8 @@ class PitiviMainWindow(gtk.Window, Loggable):
         if self.settings.mainWindowHeight:
             height = self.settings.mainWindowHeight
         self.set_default_size(width, height)
+        if height == -1 and width == -1:
+            self.maximize()
         self._do_pending_fullscreen = False
         # FIXME: don't know why this doesn't work
         #if self.settings.mainWindowFullScreen:
@@ -368,7 +424,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         # toolbar buttons should be clickable with the mouse cursor at the
         # very bottom of the screen.
         ttb = self.uimanager.get_widget("/TimelineToolBar")
-        vbox.pack_start(ttb, False)
+        vbox.pack_start(ttb, expand=False)
 
         self.show_all()
 
@@ -379,14 +435,12 @@ class PitiviMainWindow(gtk.Window, Loggable):
             ttb.props.visible = False
 
         #application icon
-        self.set_icon_from_file(get_global_pixmap_dir()
-            + "/pitivi.png")
+        self.set_icon_name("pitivi")
 
     def _connectToSourceList(self):
         # FIXME: projecttabs creates the "components" but then has no API to get
         # them back
-        sourcelist = self.projecttabs._full_list[0]
-        sourcelist.connect('play', self._sourceListPlayCb)
+        self.sourcelist.connect('play', self._sourceListPlayCb)
 
     def toggleFullScreen(self):
         """ Toggle the fullscreen mode of the application """
@@ -420,23 +474,14 @@ class PitiviMainWindow(gtk.Window, Loggable):
 
 ## Missing Plugin Support
 
-    def _installPlugins(self, details):
+    def _installPlugins(self, details, missingPluginsCallback):
         context = gst.pbutils.InstallPluginsContext()
         context.set_xid(self.window.xid)
 
         res = gst.pbutils.install_plugins_async(details, context,
-                self._installPluginsAsyncCb)
+                missingPluginsCallback)
         return res
 
-    def _installPluginsAsyncCb(self, result):
-        missing_plugins, self.missing_plugins = self.missing_plugins, []
-
-        if result != gst.pbutils.INSTALL_PLUGINS_SUCCESS:
-            return
-
-        gst.update_registry()
-        self.app.current.sources.addUris(missing_plugins)
-
 ## UI Callbacks
 
     def _configureCb(self, unused_widget, event):
@@ -444,9 +489,12 @@ class PitiviMainWindow(gtk.Window, Loggable):
             self.settings.mainWindowWidth = event.width
             self.settings.mainWindowHeight = event.height
 
-    def _destroyCb(self, unused_widget, unused_data=None):
+    def _deleteCb(self, unused_widget, unused_data=None):
         self._saveWindowSettings()
-        self.app.shutdown()
+        if not self.app.shutdown():
+            return True
+
+        return False
 
     def _exposeEventCb(self, unused_widget, event):
         if self._do_pending_fullscreen:
@@ -469,7 +517,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
 ## Toolbar/Menu actions callback
 
     def _newProjectMenuCb(self, unused_action):
-        self.app.newBlankProject()
+        self.app.projectManager.newBlankProject()
 
     def _openProjectCb(self, unused_action):
 
@@ -478,6 +526,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
             action=gtk.FILE_CHOOSER_ACTION_OPEN,
             buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OPEN, gtk.RESPONSE_OK))
+        chooser.set_icon_name("pitivi")
         chooser.set_select_multiple(False)
         chooser.set_current_folder(self.settings.lastProjectFolder)
         formats = formatter.list_formats()
@@ -497,7 +546,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         if response == gtk.RESPONSE_OK:
             uri = chooser.get_uri()
             uri = unquote(uri)
-            self.app.loadProject(uri = uri)
+            self.app.projectManager.loadProject(uri)
 
         chooser.destroy()
         return True
@@ -506,13 +555,14 @@ class PitiviMainWindow(gtk.Window, Loggable):
         if not self.project.uri:
             self._saveProjectAsCb(unused_action)
         else:
-            self.project.save(overwrite=True)
+            self.app.projectManager.saveProject(self.project, overwrite=True)
 
     def _saveProjectAsCb(self, unused_action):
         uri = self._showSaveAsDialog(self.app.current)
-        if uri:
-            self.project.save(uri, overwrite=True)
-            self.project.uri = uri
+        if uri is not None:
+            return self.app.projectManager.saveProject(self.project, uri, overwrite=True)
+
+        return False
 
     def _projectSettingsCb(self, unused_action):
         from projectsettings import ProjectSettingsDialog
@@ -539,10 +589,15 @@ class PitiviMainWindow(gtk.Window, Loggable):
     def _aboutResponseCb(self, dialog, unused_response):
         dialog.destroy()
 
+    def _showWebsiteCb(self, dialog, uri):
+        import webbrowser
+        webbrowser.open_new(uri)
+
     def _aboutCb(self, unused_action):
         abt = gtk.AboutDialog()
         abt.set_name(APPNAME)
         abt.set_version("v%s" % pitivi_version)
+        gtk.about_dialog_set_url_hook(self._showWebsiteCb)
         abt.set_website("http://www.pitivi.org/")
         authors = ["Edward Hervey <bilboed@bilboed.com>",
                    "Alessandro Decina <alessandro.decina@collabora.co.uk>",
@@ -562,10 +617,17 @@ class PitiviMainWindow(gtk.Window, Loggable):
         abt.set_authors(authors)
         abt.set_license(_("GNU Lesser General Public License\n"
                           "See http://www.gnu.org/copyleft/lesser.html for more details"))
-        abt.set_icon_from_file(get_global_pixmap_dir() + "/pitivi.png")
+        abt.set_icon_name("pitivi")
+        abt.set_logo_icon_name("pitivi")
         abt.connect("response", self._aboutResponseCb)
         abt.show()
 
+    def _undoCb(self, action):
+        self.app.action_log.undo()
+
+    def _redoCb(self, action):
+        self.app.action_log.redo()
+
     def _pluginManagerCb(self, unused_action):
         from pluginmanagerdialog import PluginManagerDialog
         PluginManagerDialog(self.app.plugin_manager)
@@ -620,53 +682,150 @@ class PitiviMainWindow(gtk.Window, Loggable):
     def loop(self, unused_action):
         pass
 
-## PiTiVi main object callbacks
 
-    def _setApplication(self):
-        if self.app:
-            self.project = self.app.current
-
-    app = receiver(_setApplication)
-
-    @handler(app, "new-project-loaded")
-    def _newProjectLoadedCb(self, unused_pitivi, project):
+    def _projectManagerNewProjectLoadedCb(self, projectManager, project):
         self.log("A NEW project is loaded, update the UI!")
-        # ungrey UI
-        self.set_sensitive(True)
-
-    @handler(app, "new-project-loading")
-    def _newProjectLoadingCb(self, unused_instance, project):
-        self.log("A NEW project is being loaded, deactivate UI")
         self.project = project
-        # grey UI
-        self.set_sensitive(False)
+        self._connectToProjectSources(project.sources)
+        can_render = project.timeline.duration > 0
+        self.render_button.set_sensitive(can_render)
+        self._syncDoUndo(self.app.action_log)
 
-    @handler(app, "closing-project")
-    def _closingProjectCb(self, unused_pitivi, project):
+        if project.timeline.duration != 0:
+            self._setBestZoomRatio()
+
+    def _setBestZoomRatio(self):
+        # reset the default zoom ratio
+        Zoomable.setZoomRatio(2)
+
+        ruler_width = self.timeline.ruler.get_allocation()[2]
+        # FIXME : The problem with this algorithm is that all widget classes using
+        # the Zoomable interface will be redrawn when doing ZoomIn/Out.
+        # We should ideally figure out the optimal zoom level without redrawing widgets
+        while True:
+            current_level = Zoomable.getCurrentZoomLevel()
+
+            self.log("Setting best zoom ratio. Current level %r",
+                     current_level)
+            timeline_width = Zoomable.nsToPixel(self.project.timeline.duration)
+
+            if ruler_width == timeline_width:
+                # perfect
+                break
+
+            if ruler_width > timeline_width:
+                self.log("Trying to zoom in")
+                if current_level == len(Zoomable.zoom_levels) - 1:
+                    self.log("Reached maximum zoom level, breaking out")
+                    break
+
+                Zoomable.zoomIn()
+                timeline_width = Zoomable.nsToPixel(self.project.timeline.duration)
+                if timeline_width > ruler_width:
+                    self.log("too big, zooming out and breaking out")
+                    Zoomable.zoomOut()
+                    break
+
+            else:
+                if current_level == 0:
+                    self.log("Reached minimum zoom level, breaking out")
+                    break
+
+                self.log("zooming out")
+                Zoomable.zoomOut()
+
+    def _projectManagerNewProjectLoadingCb(self, projectManager, uri):
+        self.log("A NEW project is being loaded, deactivate UI")
+
+    def _projectManagerSaveProjectFailedCb(self, projectManager,
+            project, uri, exception):
+        # FIXME: do something here
+        self.error("failed to save project")
+
+    def _projectManagerProjectSavedCb(self, projectManager, project, uri):
+        self.app.action_log.checkpoint()
+        self._syncDoUndo(self.app.action_log)
+        if project.uri is None:
+            project.uri = uri
+
+    def _projectManagerClosingProjectCb(self, projectManager, project):
         if not project.hasUnsavedModifications():
             return True
 
-        dialog = gtk.MessageDialog(
-            self,
-            gtk.DIALOG_MODAL,
-            gtk.MESSAGE_QUESTION,
-            gtk.BUTTONS_YES_NO,
-            _("The project has unsaved changes. Do you wish to close the project?"))
+        if project.uri:
+            save = gtk.STOCK_SAVE
+        else:
+            save = gtk.STOCK_SAVE_AS
+
+        dialog = gtk.Dialog("",
+            self, gtk.DIALOG_MODAL | gtk.DIALOG_NO_SEPARATOR,
+            (_("Close without saving"), gtk.RESPONSE_REJECT,
+                    gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                    save, gtk.RESPONSE_YES))
+        dialog.set_icon_name("pitivi")
+        dialog.set_resizable(False)
+        dialog.set_has_separator(False)
+        dialog.set_default_response(gtk.RESPONSE_YES)
+
+        primary = gtk.Label()
+        primary.set_line_wrap(True)
+        primary.set_use_markup(True)
+        primary.set_alignment(0, 0.5)
+
+        message = _("Save changes to the current project before closing?")
+        primary.set_markup("<span weight=\"bold\">" + message + "</span>")
+
+        secondary = gtk.Label()
+        secondary.set_line_wrap(True)
+        secondary.set_use_markup(True)
+        secondary.set_alignment(0, 0.5)
+        secondary.props.label = _("If you don't save some of your "
+                "changes will be lost")
+
+        # put the text in a vbox
+        vbox = gtk.VBox(False, 12)
+        vbox.pack_start(primary, expand=True, fill=True)
+        vbox.pack_start(secondary, expand=True, fill=True)
+
+        # make the [[image] text] hbox
+        image = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING,
+               gtk.ICON_SIZE_DIALOG)
+        hbox = gtk.HBox(False, 12)
+        hbox.pack_start(image, expand=False)
+        hbox.pack_start(vbox, expand=True, fill=True)
+        action_area = dialog.get_action_area()
+        # FIXME: find out where this "6" comes from. It's needed to align our
+        # hbox with the action area button box
+        hbox.set_border_width(6)
+
+        # stuff the hbox in the dialog
+        content_area = dialog.get_content_area()
+        content_area.pack_start(hbox, expand=True, fill=True)
+        content_area.set_spacing(14)
+        hbox.show_all()
+
         response = dialog.run()
         dialog.destroy()
         if response == gtk.RESPONSE_YES:
-            return True
-        return False
+            if project.uri is not None:
+                res = self.app.projectManager.saveProject(project, overwrite=True)
+            else:
+                res = self._saveProjectAsCb(None)
+        elif response == gtk.RESPONSE_REJECT:
+            res = True
+        else:
+            res = False
 
-    @handler(app, "project-closed")
-    def _projectClosedCb(self, unused_pitivi, project):
+        return res
+
+    def _projectManagerProjectClosedCb(self, projectManager, project):
         # we must disconnect from the project pipeline before it is released
+        self._disconnectFromProjectSources(project.sources)
         self.viewer.setAction(None)
         self.viewer.setPipeline(None)
         return False
 
-    @handler(app, "new-project-failed")
-    def _notProjectCb(self, unused_pitivi, reason, uri):
+    def _projectManagerNewProjectFailedCb(self, projectManager, uri, exception):
         # ungrey UI
         dialog = gtk.MessageDialog(self,
             gtk.DIALOG_MODAL,
@@ -674,23 +833,31 @@ class PitiviMainWindow(gtk.Window, Loggable):
             gtk.BUTTONS_OK,
             _("PiTiVi is unable to load file \"%s\"") %
                 uri)
+        dialog.set_icon_name("pitivi")
         dialog.set_title(_("Error Loading File"))
-        dialog.set_property("secondary-text", reason)
+        dialog.set_property("secondary-text", str(exception))
         dialog.run()
         dialog.destroy()
         self.set_sensitive(True)
 
-    @handler(app, "missing-uri")
-    def _missingUriCb(self, instance, formatter, uri):
-        dialog = gtk.Dialog(_("Locate missing file..."), 
+    def _projectManagerMissingUriCb(self, instance, formatter, uri, factory):
+        dialog = gtk.Dialog(_("Locate missing file..."),
             self,
             gtk.DIALOG_MODAL,
             buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
             gtk.STOCK_OPEN, gtk.RESPONSE_OK))
+        dialog.set_icon_name("pitivi")
         dialog.set_border_width(12)
         dialog.get_content_area().set_spacing(6)
 
-        label = gtk.Label(_("Please locate the missing file, '%s'" % uri))
+        text = _("The following file has moved," +
+            " please tell PiTiVi where to find it.") + "\n\n" + \
+            beautify_factory(factory) + "\n" +\
+            _("<b>Duration:</b>") + beautify_length(factory.duration)
+
+        label = gtk.Label()
+        label.set_markup(text)
+        label.set_justify(gtk.JUSTIFY_CENTER)
         dialog.get_content_area().pack_start(label, False, False)
         label.show()
 
@@ -700,6 +867,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         dialog.get_content_area().pack_start(chooser, True, True)
         chooser.show()
 
+        dialog.set_size_request(640, 480)
         response = dialog.run()
 
         if response == gtk.RESPONSE_OK:
@@ -716,6 +884,48 @@ class PitiviMainWindow(gtk.Window, Loggable):
 
         dialog.destroy()
 
+    def _connectToProjectSources(self, sourcelist):
+        sourcelist.connect("missing-plugins", self._sourceListMissingPluginsCb)
+
+    def _disconnectFromProjectSources(self, sourcelist):
+        sourcelist.disconnect_by_func(self._sourceListMissingPluginsCb)
+
+    def _actionLogCommit(self, action_log, stack, nested):
+        if nested:
+            return
+
+        self._syncDoUndo(action_log)
+
+    def _actionLogCleaned(self, action_log):
+        self._syncDoUndo(action_log)
+
+    def _actionLogUndo(self, action_log, stack):
+        self._syncDoUndo(action_log)
+
+    def _actionLogRedo(self, action_log, stack):
+        self._syncDoUndo(action_log)
+
+    def _syncDoUndo(self, action_log):
+        undo_action = self.actiongroup.get_action("Undo")
+        can_undo = bool(action_log.undo_stacks)
+        undo_action.set_sensitive(can_undo)
+
+        dirty = action_log.dirty()
+        save_action = self.actiongroup.get_action("SaveProject")
+        save_action.set_sensitive(dirty)
+        self.app.current.setModificationState(dirty)
+
+        redo_action = self.actiongroup.get_action("Redo")
+        can_redo = bool(action_log.redo_stacks)
+        redo_action.set_sensitive(can_redo)
+
+        if self.project is not None:
+            app_name = "%s %s" % (APPNAME, pitivi_version)
+            title = u"%s \u2014 %s" % (self.project.name, app_name)
+            if dirty:
+                title = "*" + title
+            title = title.encode("utf8")
+            self.set_title(title)
 
 ## PiTiVi current project callbacks
 
@@ -734,10 +944,10 @@ class PitiviMainWindow(gtk.Window, Loggable):
             sett = self.project.getSettings()
             self.viewer.setDisplayAspectRatio(float(sett.videopar * sett.videowidth) / float(sett.videoheight))
 
-    @handler(project, "missing-plugins")
-    def _projectMissingPluginsCb(self, project, uri, detail, message):
-        self.missing_plugins.append(uri)
-        return self._installPlugins(detail)
+    def _sourceListMissingPluginsCb(self, project, uri, factory,
+            details, descriptions, missingPluginsCallback):
+        res = self._installPlugins(details, missingPluginsCallback)
+        return res
 
 ## Current Project Pipeline
 
@@ -773,6 +983,10 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.timeline.timelinePositionChanged(position)
         self.timelinepos = position
 
+    @handler(project_pipeline, "state-changed")
+    def _timelinePipelineStateChangedCb(self, pipeline, state):
+        self.timeline.stateChanged(state)
+
 ## Project Timeline (not to be confused with UI timeline)
 
     project_timeline = receiver()
@@ -795,6 +1009,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
             buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
             gtk.STOCK_SAVE, gtk.RESPONSE_OK))
 
+        chooser.set_icon_name("pitivi")
         chooser.set_select_multiple(False)
         chooser.set_current_name(_("Untitled.xptv"))
         chooser.set_current_folder(self.settings.lastProjectFolder)
@@ -845,9 +1060,9 @@ class PitiviMainWindow(gtk.Window, Loggable):
             return
 
         # Use factory from our source list if we have the given uri
-        if uri in self.project.sources:
-            fact = self.project.sources[uri]
-        else:
+        try:
+            fact = self.project.sources.getUri(uri)
+        except SourceListError:
             from pitivi.factories.file import FileSourceFactory
             fact = FileSourceFactory(uri)
         self._viewFactory(fact)
