@@ -299,7 +299,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
                 focus = self._temp_objects[0]
                 self._move_context = MoveContext(self.timeline,
                         focus, set(self._temp_objects[1:]))
-            self._move_temp_source(x, y)
+            self._move_temp_source(self.hadj.props.value + x, y)
         return True
 
     def _dragLeaveCb(self, unused_layout, unused_context, unused_tstamp):
@@ -318,7 +318,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         focus = self._temp_objects[0]
         self._move_context = MoveContext(self.timeline,
                 focus, set(self._temp_objects[1:]))
-        self._move_temp_source(x, y)
+        self._move_temp_source(self.hadj.props.value + x, y)
         self._move_context.finish()
         self.app.action_log.commit()
         context.drop_finish(True, timestamp)
@@ -368,7 +368,8 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self._position = position
         self.ruler.timelinePositionChanged(position)
         self._canvas._position = position
-        self.scrollToPlayhead()
+        if self._state == gst.STATE_PLAYING:
+            self.scrollToPlayhead()
 
     def stateChanged(self, state):
         self._state = state
@@ -382,8 +383,6 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         return False
 
     def scrollToPosition(self, position):
-        if not self._state == gst.STATE_PLAYING:
-            return
         if position > self.hadj.upper:
             # we can't perform the scroll because the canvas needs to be
             # updated
@@ -428,8 +427,8 @@ class Timeline(gtk.Table, Loggable, Zoomable):
     def _timelineStartDurationChanged(self, unused_timeline, duration):
         if self.shrink:
             self._prev_duration = duration
-            self.ruler.setMaxDuration(duration)
-            self._canvas.setMaxDuration(duration)
+            self.ruler.setMaxDuration(duration + 60 * gst.SECOND)
+            self._canvas.setMaxDuration(duration + 60 * gst.SECOND)
             self.ruler.setShadedDuration(duration)
         else:
             # only resize if new size is larger
