@@ -171,7 +171,6 @@ class PitiviViewer(gtk.VBox, Loggable):
         self.action = action
         # FIXME: fix this properly?
         self.drawingarea.action = action
-        self.drawingarea.have_set_xid = False
         dar = float(4/3)
         try:
             producer = action.producers[0]
@@ -212,7 +211,6 @@ class PitiviViewer(gtk.VBox, Loggable):
                                       obey_child=False)
         self.pack_start(self.aframe, expand=True)
         self.drawingarea = ViewerWidget(self.action)
-        self.drawingarea.connect_after("expose-event", self._drawingAreaExposeCb)
         self.aframe.add(self.drawingarea)
 
         # Slider
@@ -272,7 +270,7 @@ class PitiviViewer(gtk.VBox, Loggable):
             bbox.show_all()
             width, height = bbox.size_request()
             width += 110
-            height = width / self.aframe.props.ratio
+            height = int(width / self.aframe.props.ratio)
             self.aframe.set_size_request(width , height)
 
     def showControls(self):
@@ -306,20 +304,6 @@ class PitiviViewer(gtk.VBox, Loggable):
         except:
             self.warning("could not set ratio !")
 
-    def _drawingAreaExposeCb(self, drawingarea, event):
-        drawingarea.disconnect_by_func(self._drawingAreaExposeCb)
-        for state in range(gtk.STATE_INSENSITIVE + 1):
-            drawingarea.modify_bg(state, drawingarea.style.black)
-        self.debug("yay, we are exposed !")
-        if self.pipeline:
-            try:
-                self.pipeline.paused()
-            except:
-                self.currentState = gst.STATE_NULL
-            else:
-                self.currentState = gst.STATE_PAUSED
-        return False
-
     ## gtk.HScale callbacks for self.slider
 
     def _sliderButtonPressCb(self, slider, unused_event):
@@ -454,7 +438,8 @@ class PitiviViewer(gtk.VBox, Loggable):
         name = message.structure.get_name()
         self.log('message:%s / %s', message, name)
         if name == 'prepare-xwindow-id':
-            self.drawingarea.set_xwindow_id()
+            sink = message.src
+            sink.set_xwindow_id(self.drawingarea.window_xid)
 
 
 class ViewerWidget(gtk.DrawingArea, Loggable):
@@ -468,18 +453,13 @@ class ViewerWidget(gtk.DrawingArea, Loggable):
         gtk.DrawingArea.__init__(self)
         Loggable.__init__(self)
         self.action = action # FIXME : Check if it's a view action
-        self.have_set_xid = False
         self.unset_flags(gtk.SENSITIVE)
+        for state in range(gtk.STATE_INSENSITIVE + 1):
+            self.modify_bg(state, self.style.black)
 
-    def set_xwindow_id(self):
-        """ set the widget's XID on the configured videosink. """
-        self.log("...")
-        if self.have_set_xid:
-            return
-        self.action.set_window_xid(self.window.xid)
-        self.unset_flags(gtk.DOUBLE_BUFFERED)
-        self.have_set_xid = True
-
+    def do_realize(self):
+        gtk.DrawingArea.do_realize(self)
+        self.window_xid = self.window.xid
 
 class PlayPauseButton(gtk.Button, Loggable):
     """ Double state gtk.Button which displays play/pause """
@@ -493,6 +473,7 @@ class PlayPauseButton(gtk.Button, Loggable):
     def __init__(self):
         gtk.Button.__init__(self, label="")
         Loggable.__init__(self)
+        self.get_settings().props.gtk_button_images = True
         self.playing = True
         self.setPlay()
         self.connect('clicked', self._clickedCb)
